/* 
 * File:   main.c
 * Author: Adom Kwabena
 *
 * Created on January 16, 2015, 6:57 PM
 */

#include <xc.h>
#include <pps.h>
#include <uart.h>
#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include "TimeDelay.h"
#include "Graphics/Graphics.h"
#include "resource.h"

//Definitions
#define MAX_BUFFER_SIZE 100
#define FIELD_SIZE      10
#define XBEE_RST        _LATA0
#define XBEE_RST_TRIS   _TRISA0

//Variables
char rx_buffer[MAX_BUFFER_SIZE];
static uint8_t volatile packet_flag;
static uint8_t volatile position;

//Function Prototypes
void Parse_Packet(char *data_in, char data_out[][FIELD_SIZE], uint8_t *num_fields);
void Update_Display(char tokens[][FIELD_SIZE], uint8_t token_count);
void System_Init();

void __attribute__((interrupt, shadow, no_auto_psv)) _U1RXInterrupt(void){
    IFS0bits.U1RXIF = 0;
    
    while(DataRdyUART1()){
        char tmp = ReadUART1();
        if(tmp == '\n'){
            packet_flag = true;
            rx_buffer[position] = '\0';
            position = 0;
        }else{
            if(position == (MAX_BUFFER_SIZE - 1)){
                position = 0;
            }
            WriteUART1(tmp);
            rx_buffer[position++] = tmp;
        }
    }
}

void Parse_Packet(char *data_in, char data_out[][FIELD_SIZE], uint8_t *num_fields){
    char *ptr;
    char *tokens = ",";
    
    *num_fields = 0;
    ptr = strtok(data_in, tokens);
   
    while(ptr){
        strcpy(data_out[(*num_fields)++], ptr);
        ptr = strtok(NULL, tokens);
    }
    
    packet_flag = false;
}

void Update_Display(char tokens[][FIELD_SIZE], uint8_t token_count){
    char s[80];
    float temperature;
    float humidity;
    float air_quality;
    int i;
    int height;
    
    SetColor(BRIGHTRED);
    SetFont((void*)&GOLFontDefault);
    height = GetTextHeight((void*)&GOLFontDefault);
    
    for(i = 0; i < token_count; i++){
        switch(tokens[i][0]){
            case 'T':
                temperature = atof(&tokens[i][2]);
                break;
            case 'H':
                humidity = atof(&tokens[i][2]);
                break;
            case 'A':
                air_quality = atof(&tokens[i][2]);
                break;
        }
    }
    
    OutTextXY(160, 0, "Remote Display");
    
    sprintf(s, "T:%.2f C", (double) temperature);
    OutTextXY(160 , height, s);
    
    sprintf(s, "H:%.2f %%", (double) humidity);
    OutTextXY(160, (2 * height), s);
    
    sprintf(s, "A:%.1f V", (double) air_quality);
    OutTextXY(160, (3 * height), s);
}


void System_Init(void){
    //disable uart interrupt
    CloseUART1();
    ConfigIntUART1(UART_RX_INT_EN & UART_RX_INT_PR1 &\
                    UART_TX_INT_DIS & UART_TX_INT_PR4);
    
    DisableIntU1RX;
    // Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
    //Fosc = 8MHz * 80/(4*2) = 60MHz for 8MHz input clock
    PLLFBD = 78;                    // M = (PLLFDB+2)
    CLKDIVbits.PLLPOST = 1;         // N2=4
    CLKDIVbits.PLLPRE = 0;          // N1=2
    OSCTUN = 0;                     // Tune FRC oscillator, if FRC is used
    
    ANSELA = 0x00;
    ANSELB = 0x00;
    ANSELC = 0x00;
    ANSELD = 0x00;
    ANSELE = 0x00;
    ANSELG = 0x00;

    //Initializations
    PPSUnLock;
    //configure UART1
    iPPSInput(IN_FN_PPS_U1RX, IN_PIN_PPS_RP100);
    iPPSOutput(OUT_PIN_PPS_RP101, OUT_FN_PPS_U1TX);
    
    OpenUART1(UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE &\
              UART_MODE_SIMPLEX & UART_UEN_00 & UART_DIS_WAKE &\
              UART_DIS_LOOPBACK & UART_DIS_ABAUD &UART_NO_PAR_8BIT &\
              UART_BRGH_SIXTEEN & UART_1STOPBIT,
              UART_INT_TX_BUF_EMPTY & UART_IrDA_POL_INV_ZERO &\
              UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE &\
              UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR,
              259);
    
    DelayMs(100);
    InitGraph();
    SetColor(WHITE);
    ClearDevice();
    DisplayBacklightOn();
}

int main(void) {
    
    uint8_t k;
    char tokens [5][10];
    uint8_t token_count = 0;
    
    //int width;
    //char s[] = "Fonts and Bitmaps";

    //Init
    //InitGraph();
    //SetColor(WHITE);
    //ClearDevice();
    //DisplayBacklightOn();
    System_Init();
    
    //Display centered banner
    //SetColor(BRIGHTRED);
    //SetFont((void*)&GOLFontDefault);
    //width = GetTextWidth(s, (void*)&GOLFontDefault);
    //OutTextXY((GetMaxX() - width) >> 1, 0, s);

    //Display bitmap
    PutImage(0, 0, (void*)&reese, IMAGE_NORMAL);
    SetColor(BRIGHTRED);
    SetFont((void*)&GOLFontDefault);
    int h = GetTextHeight((void*)&GOLFontDefault);
    
    OutTextXY(160, 0, "Remote Display");
    OutTextXY(160 , h, "Remote Display");
    OutTextXY(160, (2 * h), "Remote Display");
    OutTextXY(160, (3 * h), "Remote Display");
    for(k = 0; k < 6; k++){
        DelayMs(500);
    }
    SetColor(WHITE);
    ClearDevice();
  
    //Display folder bitmap
    //PutImage(160, 60, (void*)&folder, IMAGE_NORMAL);

    EnableIntU1RX;
    //Main loop
    while(1){
        putsUART1("Hello there!\r\n");
        for(k = 0; k < 2; k++){
            DelayMs(500);
        }
        
        if(packet_flag){
            Parse_Packet(rx_buffer, tokens, &token_count);
            Update_Display(tokens, token_count);
        }
    }
}

